#include "config.h"
#include "ui_config.h"
#include "netdiscovery.h"
#include "ftpserver.h"
#include <QSettings>
#include <QHostInfo>
#include <QFileDialog>
#include <QTranslator>
#include "sharedfolders.h"

Config *Config::pInstance = NULL;

Config *Config::instance()
{
	if (pInstance)
		return pInstance;
	return new Config;
}

void Config::deleteInstance()
{
	if (pInstance)
		pInstance->deleteLater();
}

Config::Config(QWidget *parent) :
    QDialog(parent),
	QMutex(QMutex::Recursive),
    m_ui(new Ui::Config)
{
	pInstance = this;

	m_ui->setupUi(this);

	connect(m_ui->anonymous, SIGNAL(toggled(bool)), m_ui->login, SLOT(setDisabled(bool)));
	connect(m_ui->anonymous, SIGNAL(toggled(bool)), m_ui->password, SLOT(setDisabled(bool)));
	connect(m_ui->anonymous, SIGNAL(toggled(bool)), m_ui->login_label, SLOT(setDisabled(bool)));
	connect(m_ui->anonymous, SIGNAL(toggled(bool)), m_ui->password_label, SLOT(setDisabled(bool)));
	connect(m_ui->delay, SIGNAL(valueChanged(int)), NetDiscovery::pInstance, SLOT(setNotifyRate(int)));
	connect(m_ui->enableServer, SIGNAL(toggled(bool)), this, SLOT(updateServerStatus()));
	connect(m_ui->port, SIGNAL(valueChanged(int)), this, SLOT(updateServerStatus()));
	connect(m_ui->anonymous, SIGNAL(toggled(bool)), this, SLOT(updateServerStatus()));
	connect(m_ui->bAdd, SIGNAL(clicked()), this, SLOT(addFolder()));
	connect(m_ui->bRemove, SIGNAL(clicked()), this, SLOT(removeFolder()));

	loadSettings();

	connect(m_ui->anonymous, SIGNAL(toggled(bool)), this, SLOT(saveSettings()));
	connect(m_ui->delay, SIGNAL(valueChanged(int)), this, SLOT(saveSettings()));
	connect(m_ui->password, SIGNAL(textChanged(QString)), this, SLOT(saveSettings()));
	connect(m_ui->login, SIGNAL(textChanged(QString)), this, SLOT(saveSettings()));
	connect(m_ui->enableServer, SIGNAL(toggled(bool)), this, SLOT(saveSettings()));
	connect(m_ui->port, SIGNAL(valueChanged(int)), this, SLOT(saveSettings()));
	connect(m_ui->hostname, SIGNAL(textChanged(QString)), this, SLOT(saveSettings()));

	if (m_ui->enableServer->isChecked())
		FtpServer::startServer();

	loadFolders();
	loadLanguages();
	connect(m_ui->sharedFolders, SIGNAL(itemChanged(QTableWidgetItem*)), this, SLOT(editFolder(QTableWidgetItem*)));
	connect(m_ui->sharedFolders, SIGNAL(itemClicked(QTableWidgetItem*)), this, SLOT(editPath(QTableWidgetItem*)));
	connect(m_ui->translation, SIGNAL(currentIndexChanged(int)), this, SLOT(changeLanguage()));
}

Config::~Config()
{
    delete m_ui;
	pInstance = NULL;
}

void Config::changeEvent(QEvent *e)
{
    QDialog::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
		saveSettings();
        m_ui->retranslateUi(this);
		loadLanguages();
		loadSettings();
        break;
    default:
        break;
    }
}

bool Config::isAnonymous()
{
	QMutexLocker mLock(this);
	return m_ui->anonymous->isChecked();
}

QString Config::getPassword()
{
	QMutexLocker mLock(this);
	return m_ui->password->text();
}

QString Config::getLogin()
{
	QMutexLocker mLock(this);
	return m_ui->login->text();
}

QString Config::getHostname()
{
	QMutexLocker mLock(this);
	return m_ui->hostname->text();
}

int Config::getServerPort()
{
	QMutexLocker mLock(this);
	return m_ui->port->value();
}

void Config::updateServerStatus()
{
	QMutexLocker mLock(this);
	if (m_ui->enableServer->isChecked())
		FtpServer::restartServer();
	else
		FtpServer::stopServer();
}

void Config::saveSettings()
{
	QMutexLocker mLock(this);
	QSettings settings;
	settings.beginGroup("config");
	settings.setValue("anonymous", m_ui->anonymous->isChecked());
	settings.setValue("hostname", m_ui->hostname->text());
	settings.setValue("password", m_ui->password->text());
	settings.setValue("login", m_ui->login->text());
	settings.setValue("port", m_ui->port->value());
	settings.setValue("delay", m_ui->delay->value());
	settings.setValue("enableServer", m_ui->enableServer->isChecked());
	settings.endGroup();
}

void Config::loadSettings()
{
	QMutexLocker mLock(this);
	QSettings settings;
	settings.beginGroup("config");
	m_ui->anonymous->setChecked(settings.value("anonymous",true).toBool());
	m_ui->hostname->setText(settings.value("hostname", QHostInfo::localHostName()).toString());
	m_ui->password->setText(settings.value("password","password").toString());
	m_ui->login->setText(settings.value("login","login").toString());
	m_ui->port->setValue(settings.value("port",2000).toInt());
	m_ui->delay->setValue(settings.value("delay",10).toInt());
	m_ui->enableServer->setChecked(settings.value("enableServer",true).toBool());
	settings.endGroup();
}

void Config::addFolder()
{
	QMutexLocker mLock(this);
	int newIdx = m_ui->sharedFolders->rowCount();
	m_ui->sharedFolders->setRowCount(newIdx + 1);
	QString name = tr("new folder #") + QString::number(newIdx);
	folderNames << name;
	QTableWidgetItem *item;
	m_ui->sharedFolders->setItem(newIdx, 0, item = new QTableWidgetItem(QIcon(":/icons/computer.png"), name));
	item->setFlags(Qt::ItemIsEditable | Qt::ItemIsEnabled | Qt::ItemIsSelectable);

	m_ui->sharedFolders->setItem(newIdx, 1, item = new QTableWidgetItem(tr("forbidden")));
	item->setFlags(Qt::ItemIsUserCheckable | Qt::ItemIsEnabled | Qt::ItemIsSelectable);
	item->setCheckState(Qt::Unchecked);

	m_ui->sharedFolders->setItem(newIdx, 2, item = new QTableWidgetItem(""));
	item->setFlags(Qt::ItemIsEditable | Qt::ItemIsEnabled | Qt::ItemIsSelectable);

	SharedFolders::instance()->addFolder(name, SharedFolders::Folder("",false));
}

void Config::removeFolder()
{
	QMutexLocker mLock(this);
	QList<QTableWidgetItem*> list = m_ui->sharedFolders->selectedItems();
	if (list.isEmpty())
		return;
	folderNames.removeAt(list.front()->row());
	SharedFolders::instance()->removeFolder(m_ui->sharedFolders->itemAt(0, list.front()->row())->text());
	m_ui->sharedFolders->removeRow(list.front()->row());
}

void Config::editFolder(QTableWidgetItem *item)
{
	if (item == NULL || item->row() == -1)
		return;

	QMutexLocker mLock(this);

	QMap<QString, SharedFolders::Folder> folders = SharedFolders::instance()->getFolderList();
	QString name = folderNames[ item->row() ];
	SharedFolders::Folder folder = folders[ name ];
	SharedFolders::instance()->removeFolder( name );
	switch (item->column())
	{
	case 0:
		name = item->text();
		folderNames[item->row()] = name;
		break;
	case 1:
		folder.setWritable(item->checkState() == Qt::Checked);
		item->setText(folder.isWritable() ? tr("allowed") : tr("forbidden"));
		break;
	case 2:
		folder.setPath(item->text());
		break;
	};
	SharedFolders::instance()->addFolder(name, folder);
}

void Config::editPath(QTableWidgetItem *item)
{
	if (item == NULL || item->row() == -1)
		return;

	QMutexLocker mLock(this);

	QMap<QString, SharedFolders::Folder> folders = SharedFolders::instance()->getFolderList();
	QString name = folderNames[ item->row() ];
	SharedFolders::Folder folder = folders[ name ];
	SharedFolders::instance()->removeFolder( name );
	switch (item->column())
	{
	case 2:
		item->setText(QFileDialog::getExistingDirectory(this,tr("Select a folder")));
		folder.setPath(item->text());
		break;
	};
	SharedFolders::instance()->addFolder(name, folder);
}

void Config::loadFolders()
{
	QMutexLocker mLock(this);
	m_ui->sharedFolders->setRowCount(SharedFolders::instance()->nbFolders());
	folderNames.clear();
	QMap<QString, SharedFolders::Folder> folders = SharedFolders::instance()->getFolderList();
	int n = 0;
	for(QMap<QString, SharedFolders::Folder>::iterator i = folders.begin() ; i != folders.end() ; ++i)
	{
		folderNames << i.key();
		QTableWidgetItem *item;
		m_ui->sharedFolders->setItem(n, 0, item = new QTableWidgetItem(QIcon(":/icons/computer.png"), i.key()));
		item->setFlags(Qt::ItemIsEditable | Qt::ItemIsEnabled | Qt::ItemIsSelectable);

		m_ui->sharedFolders->setItem(n, 1, item = new QTableWidgetItem(i.value().isWritable() ? tr("allowed") : tr("forbidden")));
		item->setFlags(Qt::ItemIsUserCheckable | Qt::ItemIsEnabled | Qt::ItemIsSelectable);
		item->setCheckState(i.value().isWritable() ? Qt::Checked : Qt::Unchecked);

		m_ui->sharedFolders->setItem(n, 2, item = new QTableWidgetItem(i.value().getPath()));
		item->setFlags(Qt::ItemIsEditable | Qt::ItemIsEnabled | Qt::ItemIsSelectable);
		++n;
	}
}

extern QTranslator *qShareTranslator;
extern QTranslator *qQtTranslator;

void Config::changeLanguage()
{
	QString code = m_ui->translation->itemData(m_ui->translation->currentIndex()).toString();
	QSettings settings;
	settings.setValue("locale", code);
	qShareTranslator->load(":/i18n/qshare_" + code + ".qm");
	qQtTranslator->load(":/i18n/qt_" + code + ".qm");
}

void Config::loadLanguages()
{
	disconnect(m_ui->translation, SIGNAL(currentIndexChanged(int)), this, SLOT(changeLanguage()));
	int idx = m_ui->translation->currentIndex();
	m_ui->translation->clear();
	m_ui->translation->addItem(tr("french"), "fr");
	m_ui->translation->addItem(tr("english"), "en");
	m_ui->translation->addItem(tr("spanish"), "es");
	m_ui->translation->addItem(tr("russian"), "ru");
	if (idx == -1)
	{
		QString code = QSettings().value("locale", QLocale::system().name().left(2)).toString();
		for(int i = 0 ; idx == -1 && i < m_ui->translation->count() ; ++i)
			if (m_ui->translation->itemData(i) == code)
				idx = i;
	}
	m_ui->translation->setCurrentIndex(idx);
	connect(m_ui->translation, SIGNAL(currentIndexChanged(int)), this, SLOT(changeLanguage()));
}
