#include "ftpserver.h"
#include "config.h"
#include "netdiscovery.h"
#include "serverthread.h"
#include "stats.h"
#include <QDebug>

FtpServer *FtpServer::pInstance = NULL;

void FtpServer::startServer()
{
	if (pInstance)
		delete pInstance;
	pInstance = new FtpServer(Config::instance()->getServerPort());
}

void FtpServer::stopServer()
{
	if (pInstance)
		delete pInstance;
	pInstance = NULL;
}

void FtpServer::restartServer()
{
	stopServer();
	startServer();
}

bool FtpServer::serverRunning()
{
	return pInstance != NULL;
}

FtpServer::FtpServer(int port)
{
	pInstance = this;

	connect(this, SIGNAL(newThread()), Stats::instance(), SLOT(updateStats()));
	connect(this, SIGNAL(closing()), Stats::instance(), SLOT(updateStats()));
	connect(this, SIGNAL(removeThread()), Stats::instance(), SLOT(updateStats()));
	sock = new QTcpServer(this);
	sock->listen(QHostAddress::Any, port);
	connect(sock,SIGNAL(newConnection()),this,SLOT(addConnection()));

	totalSent = 0;
	totalReceived = 0;
	maxConnections = 0;
	totalConnections = 0;
}

FtpServer::~FtpServer()
{
	emit closing();
	pInstance = NULL;
}

QList<FtpServer::ConnectionInfo> FtpServer::getInfo()
{
	QList<ConnectionInfo> lInfo;
	foreach(QObject *obj, children())
	{
		ServerThread *pThread = dynamic_cast<ServerThread*>(obj);
		if (!pThread)
			continue;
		ConnectionInfo info;
		info.user = pThread->getLogin();
		info.address = pThread->getAddress();
		switch(pThread->getMode())
		{
		case ServerThread::NONE:
			info.activity.clear();
			break;
		case ServerThread::RETR:
			info.activity = "RETR " + pThread->getFilename();
			break;
		case ServerThread::STOR:
			info.activity = "STOR " + pThread->getFilename();
			break;
		case ServerThread::APPE:
			info.activity = "APPE " + pThread->getFilename();
			break;
		case ServerThread::FIND:
			info.activity = "FIND " + pThread->getFilename();
			break;
		};
		lInfo << info;
	}
	return lInfo;
}

void FtpServer::addConnection()
{
	while (sock->hasPendingConnections())
	{
		QTcpSocket *s = sock->nextPendingConnection();
		if (s)
		{
			ServerThread *thread = new ServerThread(s, this);
			connect(thread,SIGNAL(destroyed()),this,SLOT(killThread()));
			++totalConnections;
		}
	}
	maxConnections = qMax(maxConnections, nbConnections());
	emit newThread();
}

void FtpServer::killThread()
{
	emit removeThread();
}

int FtpServer::nbConnections()
{
	int n(0);
	QObjectList list = children();
	foreach(QObject *obj, list)
	{
		if (dynamic_cast<ServerThread*>(obj))
			++n;
	}
	return n;
}

QList<FtpServer::ConnectionInfo> FtpServer::getClientsInfo()
{
	if (pInstance)
		return pInstance->getInfo();
	return QList<ConnectionInfo>();
}

void FtpServer::statsSend(int n)
{
	totalSent += n;
}

void FtpServer::statsRecv(int n)
{
	totalReceived += n;
}

QStringList FtpServer::getServerInfo()
{
	QStringList lInfo;
	if (!pInstance)
	{
		lInfo << QString();
		lInfo << QString();
		lInfo << QString();
		lInfo << QString();
		lInfo << QString();
		return lInfo;
	}
	lInfo << QString::number(pInstance->totalSent);
	lInfo << QString::number(pInstance->totalReceived);
	lInfo << QString::number(pInstance->nbConnections());
	lInfo << QString::number(pInstance->maxConnections);
	lInfo << QString::number(pInstance->totalConnections);
	return lInfo;
}
