#include "netdiscovery.h"
#include <QUdpSocket>
#include <QStringList>
#include "config.h"

NetDiscovery *NetDiscovery::pInstance = NULL;

NetDiscovery::NetDiscovery() : mutex(QMutex::Recursive)
{
	stop = false;
	notifyRate = 10;
	refreshForced = false;
	pInstance = this;
	start(LowestPriority);
}

void NetDiscovery::run()
{
	mServer.clear();
	emit serverListChanged();

	QUdpSocket sock;

	sock.bind(2000);

	int n = notifyRate - 1;
	while(!stop)
	{
		QThread::sleep(1);
		++n;
		while(sock.hasPendingDatagrams())
		{
			char buf[1024];
			QHostAddress host;
			memset(buf, 0, sizeof(buf));
			sock.readDatagram(buf, 1024, &host);
			setOrUpdate(buf, host);
		}

		if (n >= notifyRate || refreshForced)
		{
			refreshForced = false;
			QString msg = Config::instance()->getHostname() + '\n' + QString::number(Config::instance()->getServerPort());
			sock.writeDatagram(msg.toAscii(), QHostAddress(QHostAddress::Broadcast), 2000);
			n = 0;
		}
	}
	stop = false;
}

void NetDiscovery::terminate()
{
	stop = true;
}

void NetDiscovery::setOrUpdate(const QString& data, const QHostAddress &host)
{
	QStringList params = data.split('\n', QString::SkipEmptyParts);
	if (params.size() < 2)
		return;

	Computer server;
	server.setHostname(host.toString());
	server.setName(params[0]);
	server.setPort(params[1].toInt());

	bool changed = false;
	mutex.lock();
	QString key = host.toString();
	changed = !mServer.contains(key) || mServer[key] != server;
	mServer[key] = server;
	mutex.unlock();
	if (changed)
		emit serverListChanged();
}

void NetDiscovery::remove(const QHostAddress &host)
{
	mutex.lock();
	if (mServer.contains(host.toString()))
	{
		mServer.remove(host.toString());
		mutex.unlock();
		emit serverListChanged();
	}
	else
		mutex.unlock();
}

QList<Computer> NetDiscovery::getServerList()
{
	QList<Computer> lServer;

	mutex.lock();
	for(QMap<QString, Computer>::iterator i = mServer.begin() ; i != mServer.end() ; ++i)
		lServer.push_back(i.value());
	mutex.unlock();

	return lServer;
}

void NetDiscovery::setNotifyRate(int rate)
{
	notifyRate = rate;
}

void NetDiscovery::forceRefresh()
{
	mutex.lock();
	mServer.clear();
	refreshForced = true;
	emit serverListChanged();
	mutex.unlock();
}

Computer NetDiscovery::getComputer(const QString &name)
{
	QMutexLocker mLock(&mutex);
	for(QMap<QString, Computer>::iterator i = mServer.begin() ; i != mServer.end() ; ++i)
		if (i.value().getName() == name)
			return i.value();
	return Computer();
}
