#include "path.h"
#include <QDateTime>
#include "sharedfolders.h"
#include <QDebug>
#include <iostream>

const QString Path::separator('/');

inline QString getOwner(const QFileInfo &info)
{
	return info.owner().isEmpty()
			? (info.ownerId() != uint(-2)
			   ? QString::number(info.ownerId())
			   : QString("owner"))
			: info.owner();
}

inline QString getGroup(const QFileInfo &info)
{
	return info.group().isEmpty()
			? (info.groupId() != uint(-2)
			   ? QString::number(info.groupId())
			   : QString("group"))
			: info.group();
}

Path::Path()
{
	currentPath = "/";
}

QString Path::pwd()
{
	return currentPath;
}

bool Path::cwd(const QString &path)
{
	QString absolutePath = absolute(path);
	QString realpath = SharedFolders::instance()->getRealPath(absolutePath);
	QFileInfo info(realpath);
	if (!info.isDir() || !info.exists())
		return false;

	currentPath = absolutePath;
	return true;
}

bool Path::mkd(const QString &name)
{
	QString absolutePath = absolute(name);
	// You can't write in the virtual root
	if (SharedFolders::getParentPath(absolutePath) == separator)
		return false;

	if (!SharedFolders::instance()->isWritable(absolutePath))
		return false;
	QDir qdir(SharedFolders::instance()->getRealPath(SharedFolders::getParentPath(absolutePath)));
	return qdir.mkdir(SharedFolders::getFileName(name));
}

void Path::chmod(const QString &path, QByteArray perms)
{
	QString absolutePath = absolute(path);
	// You can't write in the virtual root
	if (SharedFolders::getParentPath(absolutePath) == separator)
		return;

	if (!SharedFolders::instance()->isWritable(absolutePath))
		return;
	QString realname = SharedFolders::instance()->getRealPath(absolutePath);
	if (perms.size() <= 3)
	{
		perms = QByteArray(3 - perms.size(), '0') + perms;

		QFile::Permissions rights;
		// Owner rights
		int code = perms[0] - '0';
		if (code & 1)
			rights |= QFile::ExeOwner;
		if (code & 2)
			rights |= QFile::WriteOwner;
		if (code & 4)
			rights |= QFile::ReadOwner;

		// Group rights
		code = perms[1] - '0';
		if (code & 1)
			rights |= QFile::ExeGroup;
		if (code & 2)
			rights |= QFile::WriteGroup;
		if (code & 4)
			rights |= QFile::ReadGroup;

		// Other rights
		code = perms[2] - '0';
		if (code & 1)
			rights |= QFile::ExeOther;
		if (code & 2)
			rights |= QFile::WriteOther;
		if (code & 4)
			rights |= QFile::ReadOther;
		QFile::setPermissions(realname, rights);
	}
}

bool Path::rmd(const QString &name)
{
	QString absolutePath = absolute(name);
	// You can't write in the virtual root
	if (SharedFolders::getParentPath(absolutePath) == separator)
		return false;

	if (!SharedFolders::instance()->isWritable(absolutePath))
		return false;
	QDir qdir(SharedFolders::instance()->getRealPath(currentPath));
	return qdir.rmdir(SharedFolders::getFileName(name));
}

bool Path::dele(const QString &name)
{
	QString absolutePath = absolute(name);
	// You can't write in the virtual root
	if (SharedFolders::getParentPath(absolutePath) == separator)
		return false;

	if (!SharedFolders::instance()->isWritable(absolutePath))
		return false;
	QString realname = SharedFolders::instance()->getRealPath(absolutePath);
	QFileInfo info(realname);
	if (!info.isFile())
		return false;
	QDir qdir(SharedFolders::instance()->getRealPath(currentPath));
	return qdir.remove(realname);
}

bool Path::cdup()
{
	if (currentPath == separator)
		return false;
	currentPath = SharedFolders::getParentPath(currentPath);
	return true;
}

bool Path::rename(const QString &from, const QString &to)
{
	QString realFrom;
	QString realTo;
#define getRealName(__TO__, __FROM__) \
	{\
		QString absolutePath = absolute(__FROM__); \
		if (SharedFolders::getParentPath(absolutePath) == separator) \
			return false;\
		if (!SharedFolders::instance()->isWritable(absolutePath))\
			return false;\
		__TO__ = SharedFolders::instance()->getRealPath(absolutePath);\
	}
	getRealName(realFrom,from);
	getRealName(realTo,to);

	QDir qdir(SharedFolders::instance()->getRealPath(currentPath));
	return qdir.rename(realFrom, realTo);
}

void Path::list(QList<QByteArray> &sList, const QString &filename, bool utf8)
{
	QString absoluteFilename = absolute(filename);
	if (SharedFolders::getParentPath(absoluteFilename) == separator)
	{
		QString curpath = currentPath;
		if (absoluteFilename.isEmpty() || absoluteFilename[0] != '-')
			currentPath = absoluteFilename;
		list(sList, utf8);
		currentPath = curpath;
		return;
	}

	QString realfilename = SharedFolders::instance()->getRealPath(absoluteFilename);
	QFileInfo info(realfilename);
	if (info.isDir())
	{
		QString curpath = currentPath;
		currentPath = filename;
		list(sList, utf8);
		currentPath = curpath;
		return;
	}
	if (!info.exists())
	{
		list(sList, utf8);
		return;
	}

	QString currentYear = QDateTime::currentDateTime().toString("yyyy");
	sList.clear();
	QByteArray line;
	line += info.isDir() ? "d" : "-";
	line += (info.permissions() & QFile::ReadOwner) ? "r" : "-";
	line += (info.permissions() & QFile::WriteOwner) ? "w" : "-";
	line += (info.permissions() & QFile::ExeOwner) ? "x" : "-";
	line += (info.permissions() & QFile::ReadGroup) ? "r" : "-";
	line += (info.permissions() & QFile::WriteGroup) ? "w" : "-";
	line += (info.permissions() & QFile::ExeGroup) ? "x" : "-";
	line += (info.permissions() & QFile::ReadOther) ? "r" : "-";
	line += (info.permissions() & QFile::WriteOther) ? "w" : "-";
	line += (info.permissions() & QFile::ExeOther) ? "x" : "-";

	QString owner = getOwner(info);
	QString group = getGroup(info);

	line += "   1 ";
	line += owner;
	line += " ";
	QString size = QString::number(info.size());
	line += QString(8 - owner.length(), ' ');
	line += group;
	line += QString(17 - group.length() - size.length(), ' ');
	line += size;
	line += " ";
	const char *cmonth[] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };
	int monthID = qMin( 12, qMax(0, info.created().toString("M").toInt() - 1) );
	QString month = cmonth[monthID];
	QString day = info.created().toString("d");
	QString year = info.created().toString("yyyy") == currentYear
				   ? info.created().toString("hh:mm")
				   : info.created().toString("yyyy");
	line += month;
	line += QString(6 - day.length(), ' ');
	line += day;
	line += QString(6 - year.length(), ' ');
	line += year;
	line += " ";
	if (utf8)
		line += info.fileName().toUtf8();
	else
		line += info.fileName().toAscii();

	sList << line;
}

void Path::list(QList<QByteArray> &sList, bool utf8)
{
	QString realpath = SharedFolders::instance()->getRealPath(currentPath);
	QDir qdir(realpath);
	QFileInfoList fileList = qdir.entryInfoList(QDir::AllEntries, QDir::Name | QDir::DirsFirst | QDir::IgnoreCase);
	QList<QByteArray> replacementNames;

	if (realpath == separator)
	{
		fileList.clear();
		QMap<QString, SharedFolders::Folder> folders = SharedFolders::instance()->getFolderList();
		for(QMap<QString, SharedFolders::Folder>::iterator i = folders.begin() ; i != folders.end() ; ++i)
		{
			QFileInfo info(i.value().getPath());
			fileList << info;
			if (utf8)
				replacementNames << i.key().toUtf8();
			else
				replacementNames << i.key().toAscii();
		}
	}

	QString currentYear = QDateTime::currentDateTime().toString("yyyy");
	sList.clear();
	int maxSizeLength(0);
	int maxGroupLength(0);
	int maxOwnerLength(0);
	foreach(QFileInfo info, fileList)
	{
		maxSizeLength = qMax(maxSizeLength, QString::number(info.size()).length());
		maxOwnerLength = qMax(maxOwnerLength, getOwner(info).length());
		maxGroupLength = qMax(maxGroupLength, getGroup(info).length());
	}
	foreach(QFileInfo info, fileList)
	{
		QByteArray line;
		line += info.isDir() ? "d" : "-";
		line += (info.permissions() & QFile::ReadOwner) ? "r" : "-";
		line += (info.permissions() & QFile::WriteOwner) ? "w" : "-";
		line += (info.permissions() & QFile::ExeOwner) ? "x" : "-";
		line += (info.permissions() & QFile::ReadGroup) ? "r" : "-";
		line += (info.permissions() & QFile::WriteGroup) ? "w" : "-";
		line += (info.permissions() & QFile::ExeGroup) ? "x" : "-";
		line += (info.permissions() & QFile::ReadOther) ? "r" : "-";
		line += (info.permissions() & QFile::WriteOther) ? "w" : "-";
		line += (info.permissions() & QFile::ExeOther) ? "x" : "-";

		QString owner = getOwner(info);
		QString group = getGroup(info);
		line += "   1 ";
		line += owner;
		line += " ";
		QString size = QString::number(info.size());
		line += QString(maxOwnerLength - owner.length(), ' ');
		line += group;
		line += QString(maxGroupLength + maxSizeLength + 1 - group.length() - size.length(), ' ');
		line += size;
		line += " ";
		const char *cmonth[] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };
		int monthID = qMin(qMax(info.created().toString("M").toInt() - 1, 0), 11);
		QString month = cmonth[monthID];
		QString day = info.created().toString("d");
		QString year = info.created().toString("yyyy") == currentYear
					   ? info.created().toString("hh:mm")
					   : info.created().toString("yyyy");
		line += month;
		line += QString(6 - day.length(), ' ');
		line += day;
		line += QString(6 - year.length(), ' ');
		line += year;
		line += " ";
		if (replacementNames.isEmpty())
		{
			if (utf8)
				line += info.fileName().toUtf8();
			else
				line += info.fileName().toAscii();
		}
		else
		{
			line += replacementNames.front();
			replacementNames.pop_front();
		}

		sList << line;
	}
}

void Path::nlst(QList<QByteArray> &sList, bool utf8)
{
	QString realpath = SharedFolders::instance()->getRealPath(currentPath);
	QDir qdir(realpath);
	QFileInfoList fileList = qdir.entryInfoList(QDir::AllEntries, QDir::Name | QDir::DirsFirst | QDir::IgnoreCase);

	sList.clear();
	if (realpath == separator)
	{
		QMap<QString, SharedFolders::Folder> folders = SharedFolders::instance()->getFolderList();
		for(QMap<QString, SharedFolders::Folder>::iterator i = folders.begin() ; i != folders.end() ; ++i)
		{
			if (utf8)
				sList << i.key().toUtf8();
			else
				sList << i.key().toAscii();
		}
		return;
	}

	foreach(QFileInfo info, fileList)
		if (utf8)
			sList << info.fileName().toUtf8();
		else
			sList << info.fileName().toAscii();
}

QString Path::absolute(const QString &path)
{
	QString absolutePath = path;
	if (!path.isEmpty() && path[0] != '/')
		absolutePath = currentPath + separator + path;
	QStringList keys = absolutePath.split(separator, QString::SkipEmptyParts);
	if (keys.empty())
		return separator;
	QStringList out;
	foreach(QString elt, keys)
	{
		if (elt == "..")
		{
			if (!out.isEmpty())
				out.removeLast();
		}
		else
			out << elt;
	}
	return separator + out.join(separator);
}

QString Path::realPath(const QString &path)
{
	return SharedFolders::instance()->getRealPath(absolute(path));
}

bool Path::exists(const QString &path)
{
	QString realpath = realPath(path);
	QFileInfo info(realPath(path));
	return info.exists() && realpath != separator;
}
